import os
import time
import json
import logging
import traceback
import pandas as pd
from datetime import date
from sys import executable
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver import ChromeOptions
from logging.handlers import RotatingFileHandler
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException, NoSuchElementException

# currentDirectory = os.path.dirname(executable)
currentDirectory = os.path.dirname(__file__)

def PrintLogo ():
    try:
        logoPath = os.path.join(currentDirectory, "Resources", "logo.txt")
        with open(logoPath, "r") as logo:
            print(logo.read() + "\n")
    except: logger.error("Could not read " + logoPath)

def Exit (message="", error=None, immediate=False):
    if error: message = "ERROR OCCURRED\n" + message + "\n" + traceback.format_exc()
    if message: logger.critical(message)
    if not immediate: input("Press enter to continue . . .")
    exit(1)

def main ():
    global logger
    formatter = logging.Formatter("%(asctime)s %(levelname)s %(funcName)s(%(lineno)d) %(message)s")
    fileHandler = RotatingFileHandler(os.path.join(currentDirectory, "grade.log"), mode="a", maxBytes=10*1024, backupCount=1, encoding=None, delay=0)
    fileHandler.setFormatter(formatter)
    fileHandler.setLevel(logging.INFO)
    consoleHandler = logging.StreamHandler()
    consoleHandler.setFormatter(formatter)
    consoleHandler.setLevel(logging.INFO)
    logger = logging.getLogger("root")
    logger.setLevel(logging.INFO)
    logger.addHandler(fileHandler)
    logger.addHandler(consoleHandler)
    logger.info(f"Program Started")

    PrintLogo()

    today = date.today()
    todaysDate = today.strftime("%b %d %Y")

    try:
        logger.info("Obtaining login details...")
        with open(os.path.join(currentDirectory, "login.json"), "r") as file:
            login = json.load(file)
            username = login["username"]
            password = login["password"]
            loginURL = login["loginURL"]
        if not loginURL.startswith("https://paccess.mveca.org/district/") and not loginURL.startswith("paccess.mveca.org/district/"):
            logger.critical("The \"loginURL\" parameter in login.json is incorrect. Please read the \"Provide Login Information\" step of the installation process in README.md.")
            exit(1)
    except FileNotFoundError as e:
        Exit("grade.py could not find login.json. Make sure login.json is in the same directory as this program.", e, True)
    except json.JSONDecodeError as e:
        Exit("login.json is not properly formatted.", e, True)
    except Exception as e:
        Exit("There was an error while trying to read login.json.", e, True)

    currentSchoolYear = today.year
    if today.month < 8:
        currentSchoolYear = str(currentSchoolYear - 1) + "-" + str(currentSchoolYear)
    else:
        currentSchoolYear = str(currentSchoolYear) + "-" + str(currentSchoolYear + 1)

    try:
        logger.info("Starting selenium...")
        options = ChromeOptions()
        options.add_experimental_option("detach", True)
        driver = webdriver.Chrome(options=options)
        wait = WebDriverWait(driver, 30)
        driver.get(loginURL)
        driver.maximize_window()
    except TimeoutException as e:
        logger.error("It seems you aren't connected to the internet. Try running this program again once you're connected.")
        driver.quit()
        Exit(error=e, immediate=True)
    except Exception as e:
        logger.critical("There was an error while trying to start selenium")
        driver.quit()
        Exit(error=e, immediate=True)

    try:
        logger.info("Logging into ProgressBook...")
        signInButton = driver.find_element("xpath", "//button[@id='ProgressBook']")
        signInButton.click()

        wait.until(EC.url_contains("https://central.mveca.org/auth"))

        logger.info("Entering login details...")
        usernameInput = driver.find_element("xpath", "//input[@id='user']")
        usernameInput.send_keys(username)

        passwordInput = driver.find_element("xpath", "//input[@id='password']")
        passwordInput.send_keys(password)

        loginButton = driver.find_element("xpath", "//button[@id='sign-in-button']")
        loginButton.click()

        wait.until(EC.url_contains("https://paccess.mveca.org/Student/Dashboard"))

        logger.info("Opening grades...")
        gradeButton = driver.find_element("xpath", "//a[@href='/Student/Grades']")
        gradeButton.click()
    except TimeoutException as e:
        try:
            if "invalid" in driver.find_element(By.CSS_SELECTOR, "div[ng-show='model.errorMessage']").get_attribute("innerText").lower():
                logger.critical("Invalid login credentials were entered. Check the login.json to make sure the information is correct.")
                driver.quit()
                exit(0)
        except:
            logger.error("It seems you aren't connected to the internet, or your internet is too slow. Try running this program again once you're connected.")
            driver.quit()
            Exit(error=e, immediate=True)
    except Exception as e:
        logger.critical("The program encountered an error while scrapping ProgressBook.")
        driver.quit()
        Exit(error=e, immediate=True)

    try:
        logger.info("Finding the students on file...")
        studentButtons = driver.find_elements("xpath", "//a[contains(@class,'footer-link')]")
        students = [studentButton.get_attribute("innerText") for studentButton in studentButtons]
    except NoSuchElementException as e:
        logger.critical("The program could not find the student buttons. Make sure the login detail are for a parent account not a student account.")
        driver.quit()
        Exit(error=e, immediate=True)
    except Exception as e:
        logger.critical("There was an error when trying to find the student buttons.")
        driver.quit()
        Exit(error=e, immediate=True)

    for student in students:
        logger.info("Recording Grades for: " + student)
        driver.get(driver.find_element("xpath", f"//a[contains(@class,'footer-link')][contains(text(),'{student}')]").get_attribute("href"))

        quarter = "Quarter " + driver.find_element("xpath", "//li[@class='active rptPeriod-li']").get_attribute("innerText")[3]
        gradeCSVDirectory = "Grades" + os.sep + currentSchoolYear + os.sep + quarter
        fullGradeCSVDirectory = currentDirectory + os.sep + gradeCSVDirectory
        pathToGradeCSV = fullGradeCSVDirectory + os.sep + student + ".csv"

        time.sleep(0.5)

        try:
            courseParents = driver.find_elements("xpath", "//div[contains(@class, 'active')]//tr[@class='gridrow']")
            courseNames = []
        except NoSuchElementException as e:
            logger.critical("The program was unable to find any classes")
            driver.quit()
            Exit(error=e, immediate=True)
        except Exception as e:
            logger.critical("There was an error when trying to find the student " + student + "'s classes.")
            driver.quit()
            Exit(error=e, immediate=True)

        logger.info("Editing: " + gradeCSVDirectory + os.sep + student + ".csv")
        try:
            courseInfo = pd.read_csv(pathToGradeCSV).to_dict()
            courseNames = list(courseInfo.keys())[1:]
        except FileNotFoundError:
            gradeCSVDirectory = gradeCSVDirectory.split(os.sep)
            for index in range(len(gradeCSVDirectory)):
                path = os.sep.join(gradeCSVDirectory[0:index+1])
                if not os.path.exists(currentDirectory + os.sep + path):
                    try:
                        os.mkdir(currentDirectory + os.sep + path)
                    except Exception as e:
                        logger.critical("The program encountered an error when trying to create the directory: " + currentDirectory + os.sep + path + ". Make sure the program is running with the correct permissions")
                        driver.quit()
                        Exit(error=e, immediate=True)
            courseInfo = { "Date": {} }
            for course in courseParents:
                name = course.find_element("xpath", ".//td[2]//div//a").get_attribute("innerText")
                courseNames.append(name)
                courseInfo[name] = {}
        except pd.errors.ParserError as e:
            logger.critical("It seems the " + pathToGradeCSV + " file is not formatted correctly. Fix the file and try running this program again.")
            driver.quit()
            Exit(error=e, immediate=True)
        except Exception as e:
            logger.critical("An error has occurred while trying to read " + pathToGradeCSV + ".")
            driver.quit()
            Exit(error=e, immediate=True)
            
        index = str(len(courseInfo[courseNames[0]].keys()))
        courseInfo["Date"][index] = todaysDate

        try:
            for i in range(len(courseParents)):
                gradeText = courseParents[i].find_element("xpath", ".//td[contains(@class, 'important')]//span").get_attribute("innerText")
                logger.info("Course: " + courseNames[i] + "; Grade: " + gradeText)
                gradeText = gradeText.split(" - ")[0]
                courseInfo[courseNames[i]][index] = gradeText
        except Exception as e:
            logger.critical("There was an error when recording the grades for " + pathToGradeCSV + ".")
            driver.quit()
            Exit(error=e, immediate=True)

        try:
            df = pd.DataFrame(courseInfo)
            df.to_csv(pathToGradeCSV, index=False)
        except Exception as e:
            logger.critical("There was an error when saving the grades to " + pathToGradeCSV + ".")
            driver.quit()
            Exit(error=e, immediate=True)

    driver.quit()
    logger.info("Program completed successfully.\n")

if __name__ == "__main__":
    main()