import json
from matplotlib import pyplot as plt, ticker, dates, font_manager

from load import *

def main ():
    PrintLogo()

    #* Read Config File
    with open(os.path.join(currentDirectory, "view.json"), "r") as file:
        config = json.load(file)
        view = config["view"].lower().strip()
        if view == "ask":
            try:
                while True:
                    match input("grades (g), GPA (G): "):
                        case "g" | "grade" | "grades" | "Grade" | "Grades":
                            view = "grade"
                            break
                        case "G" | "gpa" | "GPA":
                            view = "gpa"
                            break
                        case _: print("Invalid input. Use Ctrl+C to quit.")
            except KeyboardInterrupt:
                print("\nQuitting")
                exit(0)
        match view:
            case "grade" | "grades": bands = config["gradeBands"]
            case "gpa": bands = config["gpaBands"]
            case _: Exit("The value for \"view\" in view.json was set incorrectly.\nRefer to README.md for how to change \"view.\"")
        gradeLimits = config["gradeLimits"]
        gpaLimits = config["gpaLimits"]
        defaultDirectory = config["defaultDirectory"]
        courseWeights = config["courseWeights"]
        roundGrade = config["roundGrade"]
        gradeToGPA = config["gradeToGPA"]
        gradeToSignedGPA = config["gradeToSignedGPA"]
        windowColors = config["windowColors"]
        courseColors = config["courseColors"]
        gpaColors = config["gpaColors"]

    #* Retrieve a GradeCSV
    gradeCSV = Load(defaultDirectory) #? See load.py

    #* Create the plot
    plt.figure(figsize=(8, 6), facecolor=windowColors["background"])
    axis = plt.axes(facecolor=windowColors["graphBackground"])
    axis.tick_params(axis="both", which="major", labelcolor=windowColors["textColor"], colors=windowColors["graphBorder"])
    axis.tick_params(axis="both", which="minor", colors=windowColors["gridColor"])

    #* Plotting Data
    try:
        match view:
            case "grade" | "grades":
                for index, course in enumerate(gradeCSV.courses):
                    plt.plot(gradeCSV.grades["Date"], gradeCSV.grades[course], label=course, color=courseColors[index % len(courseColors)])
            case "gpa":
                gpas = { "gpa": [], "gpa+": [], "wgpa": [], "wgpa+": [] }
                def GetMaxBracket (grade, conversion):
                    conversion.sort(key=lambda x: x["min"], reverse=True)
                    for bracket in conversion:
                        if grade >= bracket["min"]:
                            return bracket
                    Exit(f"Could not convert the grade {grade} to a gpa\nCheck gradeToGPA and gradeToSignedGPA in \"view.json\"")
                saidKeyErrorForCourse = []
                for index, row in gradeCSV.grades.iterrows():
                    gpasForRow = { "gpa": [], "gpa+": [], "wgpa": [], "wgpa+": [] }
                    for course, grade in list(row.items())[1:]:
                        if np.isnan(grade): continue
                        if roundGrade: grade = round(grade)
                        gpa = GetMaxBracket(grade, gradeToGPA)["gpa"]
                        sgpa = GetMaxBracket(grade, gradeToSignedGPA)["gpa"]
                        gpasForRow["gpa"].append(gpa)
                        gpasForRow["gpa+"].append(sgpa)
                        try:
                            gpasForRow["wgpa"].append((gpa + (courseWeights[course] - 4.0)) if gpa else 0)
                            gpasForRow["wgpa+"].append((sgpa + (courseWeights[course] - 4.0)) if gpa else 0)
                        except KeyError:
                            if course not in saidKeyErrorForCourse:
                                print(f"Could not find the weight for \"{course}\". Defaulting to 4.0. To use weighted gpa, add the course weight to \"view.json\"")
                                saidKeyErrorForCourse.append(course)
                            gpasForRow["wgpa"].append(gpa)
                            gpasForRow["wgpa+"].append(sgpa)
                    for name in gpas:
                        if len(gpasForRow[name]):
                            gpas[name].append(sum(gpasForRow[name]) / len(gpasForRow[name]))
                colorIndex = 0
                gpasList = list(gpas.items())
                gpasList.sort(key=lambda x: x[0])
                for name, gpa in gpasList:
                    plt.plot(gradeCSV.grades["Date"], gpa, label=name, color=gpaColors[colorIndex % len(gpaColors)])
                    colorIndex += 1
    except Exception as e: Exit("An error occurred while trying to plot the data.", e)

    #* Customizing the graph
    #* Setting Font
    try:
        titleFont = font_manager.FontProperties(fname=os.path.join(currentDirectory, "Resources", "title.ttf"))
        textFont = font_manager.FontProperties(fname=os.path.join(currentDirectory, "Resources", "text.ttf"))
    except Exception as e: Exit("An error occurred while trying to access the fonts at " + os.path.join(currentDirectory, "Resources", "title.ttf") + " and " + os.path.join(currentDirectory, "Resources", "text.ttf"), e)

    #* Customizing Axes
    minData, maxData = axis.get_ylim()
    for band in bands:
        #? Not intended to go beyond these limits
        if band["min"] == None: band["min"] = -100000
        if band["max"] == None: band["max"] = 100000
        axis.axhspan(band["min"], band["max"], facecolor=band["color"])
    match view:
        #? The or syntax is nullish coalescing
        case "grade" | "grades": axis.set_ylim(min(minData, gradeLimits["min"] or minData), max(maxData, gradeLimits["max"] or maxData))
        case "gpa": axis.set_ylim(min(minData, gpaLimits["min"] or minData), max(maxData, gpaLimits["max"] or maxData))
    axis.yaxis.set_major_locator(ticker.MultipleLocator(5 if view == "grade" or view == "grades" else 0.1))
    axis.yaxis.set_minor_locator(ticker.MultipleLocator(1 if view == "grade" or view == "grades" else 0.02))
    axis.xaxis.set_major_locator(dates.WeekdayLocator(byweekday=dates.MO))
    axis.xaxis.set_minor_locator(dates.DayLocator())
    axis.xaxis.set_major_formatter(dates.DateFormatter("%b %d %Y"))
    axis.grid(which="major", linestyle="-", linewidth=0.2, color=windowColors["gridColor"])
    axis.grid(which="minor", linestyle="-", linewidth=0.05, color=windowColors["gridColor"])

    #* Customizing Plot and Window
    plt.title("GradeExpectations" + gradeCSV.GetCSVInfo(), color=windowColors["textColor"], pad=20, fontsize="20", fontproperties=titleFont)
    plt.xlabel("Date", color=windowColors["textColor"], fontsize="16", fontproperties=textFont)
    plt.ylabel("Grade", color=windowColors["textColor"], fontsize="16", fontproperties=textFont)
    plt.legend(loc="lower left", labelcolor=(courseColors if view == "grade" or view == "grades" else gpaColors), facecolor=windowColors["legendBackground"], edgecolor=windowColors["legendBackground"], framealpha=0.9, prop=textFont)
    plt.xticks(fontproperties=textFont)
    plt.yticks(fontproperties=textFont)
    plt.gca().set_clip_on(False)
    for spine in plt.gca().spines.values():
        spine.set_edgecolor(windowColors["graphBorder"])
    manager = plt.get_current_fig_manager()
    manager.set_window_title("GradeExpectations")
    try: manager.window.wm_iconbitmap(os.path.join(currentDirectory, "Resources", "GradeExpectations.ico"))
    except: print("There was an error while setting the GradeExpectations icon.")
    manager.window.winfo_toplevel().state("zoomed")
    plt.tight_layout()

    plt.show()

if __name__ == "__main__":
    main()